==> mule-datasense-4.3.0-20220919-sources.jar: org/mule/datasense/impl/phases/typing/resolver/batch/BatchAggregatorTypeResolver.java <==
package org.mule.datasense.impl.phases.typing.resolver.batch;

import org.mule.datasense.impl.model.annotations.DefinesTypeAnnotation;
import org.mule.datasense.impl.model.annotations.UsesTypeAnnotation;
import org.mule.datasense.impl.model.ast.MessageProcessorNode;
import org.mule.datasense.impl.model.types.EventType;
import org.mule.datasense.impl.model.types.TypeUtils;
import org.mule.datasense.impl.model.types.TypesHelper;
import org.mule.datasense.impl.phases.scoping.ExpectedAstVisitor;
import org.mule.datasense.impl.phases.scoping.ExpectedAstVisitorContext;
import org.mule.datasense.impl.phases.typing.TypingMuleAstVisitor;
import org.mule.datasense.impl.phases.typing.TypingMuleAstVisitorContext;
import org.mule.datasense.impl.phases.typing.resolver.ScopeTypeResolver;
import org.mule.metadata.message.api.MessageMetadataType;
import org.mule.metadata.message.api.MuleEventMetadataType;
import org.mule.metadata.message.api.MuleEventMetadataTypeBuilder;
import org.mule.runtime.api.component.ComponentIdentifier;
import org.mule.runtime.api.functional.Either;

public class BatchAggregatorTypeResolver extends ScopeTypeResolver {

  private static final String ATTR_STREAMING = "streaming";
  private static final String VARIABLE_RECORD = "record";
  private static final String VARIABLE_RECORDS = "records";

  public BatchAggregatorTypeResolver(ComponentIdentifier componentIdentiferScopeIn,
                                     ComponentIdentifier componentIdentiferScopeOut) {
    super(componentIdentiferScopeIn, componentIdentiferScopeOut);
  }

  @Override
  protected boolean isPropagates(MessageProcessorNode messageProcessorNode) {
    return true;
  }

  public static class ScopeIn extends ScopeTypeResolver.ScopeIn {

    @Override
    protected EventType resolve(MessageProcessorNode messageProcessorNode, EventType inputEventType,
                                TypingMuleAstVisitor typingMuleAstVisitor, TypingMuleAstVisitorContext visitorContext) {
      final EventType scopeEventType = buildInnerEventType(messageProcessorNode, inputEventType, visitorContext);
      messageProcessorNode.annotate(new UsesTypeAnnotation(new EventType()));
      messageProcessorNode.annotate(new DefinesTypeAnnotation(scopeEventType));
      return inputEventType;
    }

    private EventType buildInnerEventType(MessageProcessorNode messageProcessorNode, EventType inputEventType,
                                          TypingMuleAstVisitorContext visitorContext) {
      final MuleEventMetadataTypeBuilder muleEventMetadataTypeBuilder = TypesHelper.getMuleEventMetadataTypeBuilder();
      muleEventMetadataTypeBuilder
          .message()
          .payload().arrayType().of(TypeUtils.getMessageMetadataType(inputEventType).flatMap(MessageMetadataType::getPayloadType)
              .orElse(TypesHelper.getTypeBuilder().anyType().build()));
      muleEventMetadataTypeBuilder.message().attributes().voidType();

      final MuleEventMetadataTypeBuilder recordType = TypeUtils.asMuleEventMetadataType(inputEventType);
      if (isStreaming(messageProcessorNode)) {
        muleEventMetadataTypeBuilder.addVariable(VARIABLE_RECORD, recordType.build());
      } else {
        muleEventMetadataTypeBuilder.addVariable(VARIABLE_RECORDS,
                                                 TypesHelper.getTypeBuilder().arrayType().of(recordType).build());
      }

      final MuleEventMetadataType muleEventMetadataType = muleEventMetadataTypeBuilder.build();
      return TypeUtils.asEventType(muleEventMetadataType);
    }

    private boolean isStreaming(MessageProcessorNode messageProcessorNode) {
      // TODO EE-5755 When batch has an extension model, the default value would come from there and this wouldn't be
      // needed
      return messageProcessorNode.getComponentModel().getRawParameterValue(ATTR_STREAMING).map(Boolean::valueOf).orElse(false);
    }

    @Override
    public EventType generateExpected(MessageProcessorNode messageProcessorNode, ExpectedAstVisitor expectedAstVisitor,
                                      ExpectedAstVisitorContext visitorContext) {
      return visitorContext.getExpectedEventType();
    }
  }

}

==> mule-datasense-4.8.10-20250923.201654-1-sources.jar: org/mule/datasense/impl/phases/typing/resolver/batch/BatchAggregatorTypeResolver.java <==
package org.mule.datasense.impl.phases.typing.resolver.batch;

import static org.mule.runtime.api.meta.model.parameter.ParameterGroupModel.DEFAULT_GROUP_NAME;

import org.mule.datasense.impl.model.annotations.DefinesTypeAnnotation;
import org.mule.datasense.impl.model.annotations.UsesTypeAnnotation;
import org.mule.datasense.impl.model.ast.MessageProcessorNode;
import org.mule.datasense.impl.model.types.EventType;
import org.mule.datasense.impl.model.types.TypeUtils;
import org.mule.datasense.impl.model.types.TypesHelper;
import org.mule.datasense.impl.phases.scoping.ExpectedAstVisitor;
import org.mule.datasense.impl.phases.scoping.ExpectedAstVisitorContext;
import org.mule.datasense.impl.phases.typing.TypingMuleAstVisitor;
import org.mule.datasense.impl.phases.typing.TypingMuleAstVisitorContext;
import org.mule.datasense.impl.phases.typing.resolver.ScopeTypeResolver;
import org.mule.metadata.message.api.MessageMetadataType;
import org.mule.metadata.message.api.MuleEventMetadataType;
import org.mule.metadata.message.api.MuleEventMetadataTypeBuilder;
import org.mule.runtime.api.component.ComponentIdentifier;

public class BatchAggregatorTypeResolver extends ScopeTypeResolver {

  private static final String ATTR_STREAMING = "streaming";
  private static final String VARIABLE_RECORD = "record";
  private static final String VARIABLE_RECORDS = "records";

  public BatchAggregatorTypeResolver(ComponentIdentifier componentIdentiferScopeIn,
                                     ComponentIdentifier componentIdentiferScopeOut) {
    super(componentIdentiferScopeIn, componentIdentiferScopeOut);
  }

  @Override
  protected boolean isPropagates(MessageProcessorNode messageProcessorNode) {
    return true;
  }

  public static class ScopeIn extends ScopeTypeResolver.ScopeIn {

    @Override
    protected EventType resolve(MessageProcessorNode messageProcessorNode, EventType inputEventType,
                                TypingMuleAstVisitor typingMuleAstVisitor, TypingMuleAstVisitorContext visitorContext) {
      final EventType scopeEventType = buildInnerEventType(messageProcessorNode, inputEventType, visitorContext);
      messageProcessorNode.annotate(new UsesTypeAnnotation(new EventType()));
      messageProcessorNode.annotate(new DefinesTypeAnnotation(scopeEventType));
      return inputEventType;
    }

    private EventType buildInnerEventType(MessageProcessorNode messageProcessorNode, EventType inputEventType,
                                          TypingMuleAstVisitorContext visitorContext) {
      final MuleEventMetadataTypeBuilder muleEventMetadataTypeBuilder = TypesHelper.getMuleEventMetadataTypeBuilder();
      muleEventMetadataTypeBuilder
          .message()
          .payload().arrayType().of(TypeUtils.getMessageMetadataType(inputEventType).flatMap(MessageMetadataType::getPayloadType)
              .orElse(TypesHelper.getTypeBuilder().anyType().build()));
      muleEventMetadataTypeBuilder.message().attributes().voidType();

      final MuleEventMetadataTypeBuilder recordType = TypeUtils.asMuleEventMetadataType(inputEventType);
      if (isStreaming(messageProcessorNode)) {
        muleEventMetadataTypeBuilder.addVariable(VARIABLE_RECORD, recordType.build());
      } else {
        muleEventMetadataTypeBuilder.addVariable(VARIABLE_RECORDS,
                                                 TypesHelper.getTypeBuilder().arrayType().of(recordType).build());
      }

      final MuleEventMetadataType muleEventMetadataType = muleEventMetadataTypeBuilder.build();
      return TypeUtils.asEventType(muleEventMetadataType);
    }

    private boolean isStreaming(MessageProcessorNode messageProcessorNode) {
      return (boolean) messageProcessorNode.getComponentModel().getParameter(DEFAULT_GROUP_NAME, ATTR_STREAMING).getValue()
          .getRight();
    }

    @Override
    public EventType generateExpected(MessageProcessorNode messageProcessorNode, ExpectedAstVisitor expectedAstVisitor,
                                      ExpectedAstVisitorContext visitorContext) {
      return visitorContext.getExpectedEventType();
    }
  }

}
